@@ -7,14 +7,14 @@
 setMethod("forwardbackward","depmix",
 	function(object, return.all=TRUE, useC=TRUE, ...) {
 		fb(init=object@init,A=object@trDens,B=object@dens,ntimes=ntimes(object), 
-			stationary=object@stationary,return.all=return.all,useC=useC)
+			homogeneous=object@homogeneous,return.all=return.all,useC=useC)
 	}
 )
 
 setMethod("forwardbackward","mix",
 	function(object, return.all=TRUE, useC=TRUE, ...) {
 		fb(init=object@init,matrix(0,1,1),B=object@dens,ntimes=ntimes(object), 
-			stationary=TRUE,return.all=return.all,useC=useC)
+			homogeneous=TRUE,return.all=return.all,useC=useC)
 	}
 )
 
